package net.datastructures;
 /**
  * Implementation of the Deque interface by means of a doubly linked
  * list.  This class uses class DLNode, which implements a node of
  * the list.
  *
  * @author Natasha Gelfand
  * @author Roberto Tamassia
  */

public class NodeDeque<E> implements Deque<E>
{
  protected DLNode<E> header, trailer;  // sentinels
  protected int size;                   // number of elements

  /** Creates an empty deque. */
  public NodeDeque()
  {  // initialize an empty deque
    header = new DLNode<E>();
    trailer = new DLNode<E>();
    header.setNext(trailer);  // make header point to trailer
    trailer.setPrev(header);  // make trailer point to header
    size = 0;
  }


/**
 * Return the size of the deque, that is the number of elements it has.
 * @return  Number of elements in the deque
 */
  public int size()
  {
    return size;
  }


/**
 * This function returns true if and only if the deque is empty
 * @return  true if the deque is empty, false otherwise
 */
  public boolean isEmpty()
  {
    if (size == 0)
      return true;
    return false;
  }


/**
 * Inspect the first element without modifying the deque.
 * @return The first element in the sequence
 */
  public E getFirst() throws EmptyDequeException
  {
    if (isEmpty())
      throw new EmptyDequeException("Deque is empty.");
    return header.getNext().getElement();
  }


  public E getLast() throws EmptyDequeException
  {
    if (isEmpty())
      throw new EmptyDequeException("Deque is empty.");
    return trailer.getPrev().getElement();
  }


  public void addFirst(E o)
  {
    DLNode<E> second = header.getNext();
    DLNode<E> first = new DLNode<E>(o, header, second);
    second.setPrev(first);
    header.setNext(first);
    size++;
  }


  public void addLast(E o)
  {
    DLNode<E> secondtolast = trailer.getPrev();
    DLNode<E> last = new DLNode<E>(o, secondtolast, trailer);
    secondtolast.setNext(last);
    trailer.setPrev(last);
    size++;
  }

  public E removeFirst() throws EmptyDequeException
  {
    if (isEmpty())
      throw new EmptyDequeException("Deque is empty.");
    DLNode<E> first = header.getNext();
    E o = first.getElement();
    DLNode<E> second = first.getNext();
    header.setNext(second);
    second.setPrev(header);
    size--;
    return o;
  }


  public E removeLast() throws EmptyDequeException
  {
    if (isEmpty())
      throw new EmptyDequeException("Deque is empty.");
    DLNode<E> last = trailer.getPrev();
    E o = last.getElement();
    DLNode<E> secondtolast = last.getPrev();
    trailer.setPrev(secondtolast);
    secondtolast.setNext(trailer);
    size--;
    return o;
  }

}

